#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
""" 
A script that checks available service and see if the are running 
"""
import re
import os, sys, json
import subprocess


def run_command(command, shell=False):
    """
    Run command "command".

    :param command: The command to run. Should be a list of strings, containing the program and its argument.
    :return: Tuple containing program output and return status code.
    """
    assert isinstance(command, list)
    p = subprocess.Popen(command, shell=shell, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    stout, _ = p.communicate()

    if isinstance(stout, bytes):
        stout = stout.decode(errors='ignore') # Python 3

    return stout, p.returncode

def PsmClient(args):
    args.insert(0, '/usr/local/bluecat/PsmClient')
    out, _ = run_command(args)
    if out.find('retcode=ok') < 0:
        error_str = 'command: "PsmClient ' + ' '.join(args[1:]) + '" returned the following error: ' + out.rstrip()
        raise Exception(error_str)
    return out

known_services = {
      'dhcp-enable' : 'dhcp'
    , 'dns-enable' : 'dns'
    , 'firewall-enable' : 'firewall'
    , 'ssh-enable' : 'ssh'
}
inv_known_services = {v: k for k, v in known_services.items()}

service_to_remote_name = {
    'dns' : 'mmremote',
    'dhcp' : 'mmdhcpremote'
}

def update_mmservice_state(service_name, enable):
    systemctl_service_name = service_to_remote_name[service_name]
    if enable == '1':
        run_command(['systemctl', 'enable', systemctl_service_name])
        run_command(['systemctl', 'start', systemctl_service_name])
    else:
        run_command(['systemctl', 'disable', systemctl_service_name])
        run_command(['systemctl', 'stop', systemctl_service_name])

def get_mmservice_enabled_state(service_name):
    enabled_state =  run_command(['systemctl', 'is-enabled', service_to_remote_name[service_name]])
    if enabled_state[0].strip() == "enabled":
        return "1"
    else:
        return "0"

def get_services_psm_client_is_authoritative_for():
    # PSM client is "authoritative" for the state of the firewall/ssh on the appliance
    regexp = re.compile(r'^node \S+ (\S+)=(\d+)\s*$')
    services = {}
    services_psm_manages = [service for service in known_services if service not in ['dhcp-enable', 'dns-enable']]
    out = PsmClient(['node', 'get'])
    for line in str(out).split(os.linesep):
        re_res = regexp.split(line)
        if len(re_res) > 3:
            if re_res[1] in services_psm_manages and re_res[1]:
                services[known_services[re_res[1]]] = re_res[2]
    return services

def get_services_micetro_is_authoritative_for():
    # Micetro is "authoritative" for the state of DNS/DHCP on the appliance so we check if 
    # the mm remotes are enabled or disabled to get the state of DNS/DHCP
    services = {}
    for service in ["dns", "dhcp"]:
        services[service] = get_mmservice_enabled_state(service)
    return services

def update_service_state_in_psm(service_name, state):
    try:
        PsmClient(['node', 'set', inv_known_services[service_name] + '=' + state])
    except Exception as e:
        if "dhcpd failed to start" in str(e):
            # If dhcpd fails to start then the config is probably incorrect but the service has still been enabled
            pass
        else:
            raise

def getServices() -> dict:
    available_services = {}
    available_services.update(get_services_psm_client_is_authoritative_for())
    available_services.update(get_services_micetro_is_authoritative_for())
    return available_services

def setServices(services) -> dict:
    services_before = getServices()
    for service_name, value in services.items():
        if service_name in services_before and value != services_before.get(service_name):
            update_service_state_in_psm(service_name, value)
            if service_name in ['dns', 'dhcp']:
                update_mmservice_state(service_name, value)
                #update_firewall()  open/close firewall is possible (or easy)

    services_after = getServices()
    with open("/var/mmsuite/updater/services.conf", "w") as service_file:
        service_file.write(json.dumps(services_after))
    return services_after



if __name__ == '__main__':
    try:
        result = {}
        if (len(sys.argv) > 1) and (sys.argv[1] == '-set'):
            # input can either be an argument or a through stdin
            text = sys.argv[2] if len(sys.argv) > 2 else sys.stdin.read()
            input = json.loads(text)
            result = setServices(input)
        else:
            # assume this is get
            result = getServices()
        print(json.dumps(result))
    except Exception as e:
        sys.stderr.write(str(e) + "\n")
        sys.exit(1)


